/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.handler;

import com.gestalt.jbi.component.manager.Endpoint;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import javax.jbi.messaging.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import java.io.*;
import java.util.logging.Logger;

public abstract class AbstractMessageExchangeHandler implements Runnable {

	protected static final Logger log = Logger.getLogger(AbstractMessageExchangeHandler.class.getName());
	
	protected enum MEP {
		IN_ONLY,
	    IN_OUT,
	    IN_OPTIONAL_OUT,
	    ROBUST_IN_ONLY,
	    OUT_ONLY,
	    OUT_IN,
	    OUT_OPTIONAL_IN,
	    ROBUST_OUT_ONLY;
	    @Override
	    public String toString() {
	        switch (this) {
	        case IN_ONLY:
	            return "http://www.w3.org/2004/08/wsdl/in-only";

	        case IN_OUT:
	            return "http://www.w3.org/2004/08/wsdl/in-out";

	        case IN_OPTIONAL_OUT:
	            return "http://www.w3.org/2004/08/wsdl/in-opt-out";

	        case ROBUST_IN_ONLY:
	            return "http://www.w3.org/2004/08/wsdl/robust-in-only";

	        case OUT_ONLY:
	            return "http://www.w3.org/2004/08/wsdl/out-only";

	        case OUT_IN:
	            return "http://www.w3.org/2004/08/wsdl/out-in";

	        case OUT_OPTIONAL_IN:
	            return "http://www.w3.org/2004/08/wsdl/out-opt-in";

	        case ROBUST_OUT_ONLY:
	            return "http://www.w3.org/2004/08/wsdl/robust-out-only";

	        default:
	            return null;
	        }
	    }
	};
	
	protected DeliveryChannel channel;

	protected MessageExchange exchange;
	
	protected Endpoint endpoint;
	
	public AbstractMessageExchangeHandler(Endpoint endpoint) {
		this.endpoint = endpoint;
	}

	public void setDeliveryChannel(DeliveryChannel channel) {
		this.channel = channel;
	}

	public void setMessageExchange(MessageExchange exchange) {
		this.exchange = exchange;
	}

	public void run() {
		processMessageExchange();
	}

	public void processMessageExchange() {
		// validate exchange
		ExchangeStatus status = exchange.getStatus();
		if (ExchangeStatus.ACTIVE.equals(status)) {
			processActive();
		} else if (ExchangeStatus.DONE.equals(status)) {
			processDone();
		} else if (ExchangeStatus.ERROR.equals(status)) {
			processError(exchange.getError());
		}
	}

	private void processActive() {
		Fault fault = exchange.getFault();
		if (fault != null) {
			processFault(fault);
		} else {
			processMessage();
		}
	}
	
    protected final void send() throws MessagingException {
        send(exchange);
    }
    
    protected final void send(MessageExchange exchange) throws MessagingException {
    	channel.send(exchange);
    }

    protected final void sendSync() throws MessagingException {
        sendSync(exchange);
    }
    
    protected final void sendSync(MessageExchange exchange) throws MessagingException {
    	channel.sendSync(exchange, 30000);
    }
    
    protected final void sendSyncNoTimeout() throws MessagingException {
    	sendSyncNoTimeout(exchange);
    }
    
    protected final void sendSyncNoTimeout(MessageExchange exchange) throws MessagingException {
    	channel.sendSync(exchange);
    }
    
    protected final void sendDone() throws MessagingException {
    	sendDone(exchange);
    }
    
    protected final void sendDone(MessageExchange exchange) throws MessagingException {
    	exchange.setStatus(ExchangeStatus.DONE);
    	channel.send(exchange);
    }
    
    protected final void sendFault(MessageExchange exchange) throws MessagingException {
        Fault fault = exchange.createFault();
        sendFault(fault);
    }
    
    protected final void sendFault() throws MessagingException {
    	sendFault(exchange);
    }
    
    protected final void sendFault(Exception ex) throws MessagingException {
        sendFault(exchange, ex);
    }	
    
    protected final void sendFault(MessageExchange exchange, Exception ex) throws MessagingException {
    	Fault fault = exchange.createFault();
    	if ( ex != null ) {
    		String xmlText = getExceptionAsXmlText(ex);
    		fault.setContent(createDOMSource(new StringReader(xmlText)));
    	}
    	sendFault(fault);
    }	
    
    /**
     * return the DOM Document
     * @param xmlReader Reader
     * @return dom document
     * @throws Exception on parser exception or any other exception
     */
    public static Document buildDOMDocument(Reader xmlReader) throws Exception {
        Document xmlDoc = null;
        DocumentBuilderFactory docBuilderFactory =
            DocumentBuilderFactory.newInstance();
        docBuilderFactory.setValidating(false);
        DocumentBuilder docBuilder =
            docBuilderFactory.newDocumentBuilder();
        docBuilder.setErrorHandler( new DefaultHandler() {
            public void fatalError(SAXParseException e)
            throws SAXException {
                throw new SAXException(e.getMessage());
            }
        });
        
        docBuilder.setEntityResolver(new EntityResolver() {
            public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
                StringReader reader = new StringReader("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); // NOI18N
                InputSource source = new InputSource(reader);
                source.setPublicId(publicId);
                source.setSystemId(systemId);
                return source;
            }
        });
        
        InputSource is = new InputSource(xmlReader);
        xmlDoc = docBuilder.parse(is);
        
        return xmlDoc;
    }    
    
    /**
     * creates a DOMSource from the xml text read from the reader.
     */
    public static DOMSource createDOMSource(Reader xmlReader) {
        Document doc = null;
        try {
            doc = buildDOMDocument(xmlReader);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return new DOMSource(doc);
    }
    
    /**
     * converts the ex stracktrace to string.
     */
    public static StringBuffer getExceptionStackTrace(Exception ex) {
        StringWriter strWriter = new StringWriter();
        if ( ex != null ) {
            PrintWriter out = new PrintWriter(strWriter);
            ex.printStackTrace(out);
        }
        return strWriter.getBuffer();
    }
    
    /**
     * may be used to set the exception as fault content.
     */
    public static String getExceptionAsXmlText(Exception ex) {
        String message = ex.getMessage();
        String stackTrace = getExceptionStackTrace(ex).toString();
        String exXmlText =
            "<exception>" +
            "<message>" + message + "</message>" +
            "<stack-trace>" + stackTrace + "</stack-trace>" +
            "</exception>" ;
        return exXmlText;
    }
    
    protected void sendFault(Fault fault) throws MessagingException {
    	sendFault(exchange, fault);
    }    
    
    protected void sendFault(MessageExchange exchange, Fault fault) throws MessagingException {
    	exchange.setFault(fault);
    	channel.send(exchange);
    }    
    
    protected final void sendError(Exception ex) {
        sendError(exchange, ex);
    }    
    
    protected final void sendError(MessageExchange exchange, Exception ex) {
    	try {
    		exchange.setError(ex);
    		channel.send(exchange);
    	} catch (MessagingException msgEx) {
    		msgEx.printStackTrace();
    	}
    }    
	
    protected abstract void processError(Exception ex);
    protected abstract void processDone();
    protected abstract void processMessage();
    protected abstract void processFault(Fault fault);

}
