/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.lifecycle;

import com.gestalt.jbi.component.AbstractComponent;
import com.gestalt.jbi.component.handler.AbstractMessageExchangeHandler;
import com.gestalt.jbi.component.manager.AbstractServiceUnitManager;
import com.gestalt.jbi.component.manager.Endpoint;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AbstractMessageExchangeProcessor {

	protected static final Logger log = Logger.getLogger(AbstractMessageExchangeProcessor.class.getName());
	
	private volatile Boolean canContinue = true;
	private volatile Boolean canAccept = false;
	
	private AbstractComponent component;
	private AbstractComponentLifeCycle componentLifeCycle;
	private AbstractServiceUnitManager serviceUnitManager;
	
	private ExecutorService processorThreadPool;
	private ExecutorService handlerThreadPool;
	
	private Lock acceptMutex = new ReentrantLock();
	private Lock lifeMutex = new ReentrantLock();
	
	public AbstractMessageExchangeProcessor(AbstractComponent component) {		
		this.component = component;
		this.componentLifeCycle = (AbstractComponentLifeCycle) component.getLifeCycle();
		this.serviceUnitManager = (AbstractServiceUnitManager) component.getServiceUnitManager();
		
	}
	
	public void init() {
		handlerThreadPool = Executors.newFixedThreadPool(5);
		processorThreadPool = Executors.newSingleThreadExecutor();
		processorThreadPool.execute(new Runnable() {
			public void run() {
				while(canContinue) {
					if (canAccept) {
						processExchange();
					} else {
						try {
							Thread.sleep(2000);
						} catch (InterruptedException e) {
							
						}
					}
				}
			}
		});
	}
	
	public void shutdown() {
		lifeMutex.lock();
		canContinue = false;
		lifeMutex.unlock();
	}
	
	public void start() {
		acceptMutex.lock();
		canAccept = true;
		acceptMutex.unlock();
	}
	
	public void stop() {
		acceptMutex.lock();
		canAccept = false;
		acceptMutex.unlock();
	}
	
	private void processExchange() {
		DeliveryChannel channel = componentLifeCycle.getDeliveryChannel();
		if (channel == null) {
			return ;
		}
		try {
			MessageExchange exchange = channel.accept(3000);
			if (exchange == null) {
				log.log(Level.FINEST, "We have no message exchange");
				return ;
			}
			ServiceEndpoint serviceEndpoint = exchange.getEndpoint();
            log.fine("Received message for endpoint: " + serviceEndpoint);
            AbstractMessageExchangeHandler handler = findHandler(serviceEndpoint.getServiceName(), serviceEndpoint.getEndpointName());
			if (handler == null) {
				log.log(Level.WARNING, "No handler found for service endpoint");
				return ;
			}
			handler.setDeliveryChannel(channel);
			handler.setMessageExchange(exchange);
			handlerThreadPool.execute(handler);
		} catch (MessagingException e) {
			log.throwing(getClass().getName(), "processExchange", e);
		}
	}
	
	private AbstractMessageExchangeHandler findHandler(QName serviceName, String endpointName) {
		Endpoint endpoint = serviceUnitManager.getEndpoint(serviceName, endpointName);
		if (endpoint != null) {
			return endpoint.getProcessor();
		}
		return null;
	}
	
}
